export default class {
  // Execution flows:
  // Button -> Button.onClick() -> Utils.http.execute() -- doesn't save autofill values
  // Button -> Button.onClick() -> form.submit() -- doesn't execute Form.onSubmit(), so cannot call Utils.http.execute() there
  // Submit -> Form.onSubmit() -> Utils.http.execute() -- save autofill values and auto-submit when ENTER is pressed
  static submitData(form, component, overrideUrl = null) {
    // Analogous to Rails' form_with's `local: true`
    if (form.dataset.local) {
      // Prevent onUnload dirty prompt.
      Utils.http.clearDirtyState();
      form.submit();
    } else {
      const url = overrideUrl ? overrideUrl : form.getAttribute("action");
      const formData = new FormData(form);
      const method = form.getAttribute("method");
      if (method.toUpperCase() === "GET") {
        const data = {
          url: Utils.url.appendParams(url, formData)
        };
        Utils.http.notifyFormSubmitted();
        Utils.http.load(data, component);
      } else {
        const data = {
          url: url,
          formData: formData
        };
        Utils.http.execute(data, method, component, response => {
          Utils.http.notifyFormSubmitted();
          GLib.action.handleResponse(response, component);
        });
      }
    }
  }
}
