export default class Hash {
  constructor(object) {
    for (const key in object) {
      this[key] = object[key];
    }
  }

  remove(key) {
    const value = this[key];
    delete this[key];
    return value;
  }

  without(...keys) {
    for (const key of keys) {
      this.remove(key);
    }
    return this;
  }

  with(...keys) {
    for (const key in this) {
      if (!keys.includes(key)) {
        this.remove(key);
      }
    }
    return this;
  }

  map(handler) {
    return this.reduce((accumulator, key, value, index) => {
      accumulator[index] = handler(key, value, index);
      return accumulator;
    }, []);
  }

  reduce(handler, initialValue) {
    const keys = Object.keys(this);
    let index = 0;
    let accumulator = initialValue;
    for (const key of keys) {
      accumulator = handler(accumulator, key, this[key], index);
      index++;
    }
    return accumulator;
  }

  forEach(handler) {
    const keys = Object.keys(this);
    for (const key of keys) {
      handler(key, this[key]);
    }
  }
}
