import Vue from "vue";
import Dialog from "../nav/dialog";
import Sheet from "../nav/sheet";
import Snackbar from "../nav/snackbar";
import Vuetify from "vuetify";

export default class {
  static get dialog() {
    return LaunchDialog;
  }

  static get sheet() {
    return LaunchSheet;
  }

  static get snackbar() {
    return LaunchSnackbar;
  }
}

class LaunchDialog {
  static closestBody(component) {
    const dialog = component.$el.closest(".v-dialog");
    if (dialog) {
      return dialog.querySelector(".dialogs-body");
    }
    return null;
  }

  static _initStack() {
    if (!this.stack) {
      this.stack = [];
    }
  }

  static topDialog() {
    this._initStack();

    return this.stack.last();
  }

  static open(properties, component) {
    if (!this.stack) {
      this.stack = [];
    }

    // https://css-tricks.com/creating-vue-js-component-instances-programmatically/
    const DialogClass = Vue.extend(Dialog);
    const instance = new DialogClass({
      vuetify: new Vuetify({
        theme: {
          themes: Utils.settings.themes
        }
      }),
      propsData: {
        spec: properties,
        stack: this.stack,
        vueApp: window.vueApp
      }
    });

    if (component) {
      // const placeholder = component
      //   .$dialogContainer()
      //   .appendChild(document.createElement("div"));

      const placeholder = component.$el.appendChild(
        document.createElement("div")
      );
      instance.$mount(placeholder);
    } else {
      console.error("A dialog has to be displayed in a component");
    }
  }

  static reload(properties, component) {
    if (!this.stack) {
      return;
    }

    Utils.type.ifObject(this.stack.last(), dialog => {
      dialog.reload(properties);
    });
  }

  static close(properties, component) {
    if (!this.stack) {
      this.stack = [];
    }

    Utils.type.ifObject(this.stack.last(), dialog => {
      dialog.close();
    });

    Utils.type.ifObject(properties["onClose"], it => {
      Action.execute(it, component);
    });
  }

  // This is only meant to be used internally
  static alert(message, component) {
    const properties = {
      message: message
    };
    const spec = Object.assign({}, properties, {
      buttons: [
        {
          text: "OK",
          onClick: properties.onClose
        }
      ]
    });
    this.open(spec, component);
  }
}

class LaunchSheet {
  static open(properties, component) {
    const SheetClass = Vue.extend(Sheet);
    const instance = new SheetClass({
      vuetify: new Vuetify({
        theme: {
          themes: Utils.settings.themes
        }
      }),
      propsData: { spec: properties }
    });

    if (component) {
      const placeholder = component.$el.appendChild(
        document.createElement("div")
      );
      instance.$mount(placeholder);
    } else {
      console.error("A sheet has to be displayed in a component");
    }

    return instance;
  }

  static confirm(properties, onConfirm, component) {
    const spec = Object.assign({}, properties, {
      buttons: [
        {
          text: "Confirm",
          onClickFunction: onConfirm
        }
      ]
    });
    this.open(spec, component);
  }

  static alert(message, buttons, component) {
    const spec = { message: message, buttons: buttons };
    this.open(spec, component);
  }
}

class LaunchSnackbar {
  static open(properties, component) {
    const SnackbarClass = Vue.extend(Snackbar);
    const instance = new SnackbarClass({
      vuetify: window.vueApp.vuetify,
      propsData: { spec: properties, vueApp: window.vueApp }
    });

    if (component) {
      // This doesn't seem to work when it is triggered from a list row's menu.
      // const placeholder = component.$el.appendChild(
      //   document.createElement("div")
      // );

      const placeholder = document.body.appendChild(
        document.createElement("div")
      );
      instance.$mount(placeholder);
      instance.show = true;
    } else {
      console.error("A snackbar has to be displayed in a component");
    }

    return instance;
  }

  static error(message, component) {
    this.open({ message: message, styleClasses: ["error"] }, component);
  }

  static indicator(message, component) {
    const instance = this.open({ message: message, timeout: -1 }, component);
    instance.indicator = true;
  }
}
