// From https://stackoverflow.com/questions/47157428/how-to-implement-a-pseudo-blocking-async-queue-in-js-ts

import Vue from "vue";

// const jobQueue = [];
export default class {
  constructor() {
    // invariant: at least one of the arrays is empty
    this.resolvers = [];
    this.promises = [];
  }
  _add() {
    this.promises.push(
      new Promise(resolve => {
        this.resolvers.push(resolve);
      })
    );
  }
  enqueue(t) {
    // if (this.resolvers.length) this.resolvers.shift()(t);
    // else this.promises.push(Promise.resolve(t));
    if (!this.resolvers.length) this._add();
    this.resolvers.shift()(t);
  }
  dequeue() {
    if (!this.promises.length) this._add();
    return this.promises.shift();
  }
  // now some utilities:
  isEmpty() {
    // there are no values available
    return !this.promises.length; // this.length <= 0
  }
  isBlocked() {
    // it's waiting for values
    return !!this.resolvers.length; // this.length < 0
  }
  get length() {
    return this.promises.length - this.resolvers.length;
  }
  sleep(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
  }
  async start() {
    // TODO: This is experimental
    const delayBetweenJobs = 2000;

    for await (const job of this) {
      console.log("Executing job...");
      job();
      if (delayBetweenJobs) {
        console.log("Waiting for next job1...");
        await this.sleep(delayBetweenJobs);
        console.log("Waiting for next job2...");
      }
    }
  }
  [Symbol.asyncIterator]() {
    // Todo: Use AsyncIterator.from()
    return {
      next: () => this.dequeue().then(value => ({ done: false, value })),
      [Symbol.asyncIterator]() {
        return this;
      }
    };
  }
}

// export default class {
//   constructor() {
//     this.jobQueue = [];
//     this.timerHandle = null;
//   }
//   // static get csrfElement() {
//   //   return document.querySelector(`meta[name="csrf-token"]`);
//   // }

//   // static getCsrf() {
//   //   return this.csrfElement.getAttribute("content");
//   // }

//   // static setCsrf(value) {
//   //   return this.csrfElement.setAttribute("content", value);
//   // }

//   start(interval) {
//     this.timerHandle = setInterval(() => {
//       console.log("DEQUQE1");
//       const oldestJob = this.jobQueue.shift();
//       if (oldestJob) {
//         console.log("DEQUQE2");
//         oldestJob();
//       }
//     }, interval);
//   }

//   stop() {
//     clearInterval(this.timerHandle);
//   }

//   enqueue(job) {
//     this.jobQueue.push(job);
//   }
// }
